"""Datatable chart configuration and compilation."""

from kb_dashboard_core.panels.charts.datatable.config import ESQLDatatableChart, LensDatatableChart

__all__ = ['ESQLDatatableChart', 'LensDatatableChart']
